from datetime import datetime

from ximalaya.core.client import XimalayaClient, ResponsePaginator
from ximalaya.core.models import AlbumComment


def api_album_comment_list(client: XimalayaClient, *, album_id: int, page_size: int = 50, order: str = 'content-score-desc') -> ResponsePaginator[AlbumComment]:
    client.host = 'mobile.ximalaya.com'
    timestamp = int(datetime.now().timestamp() * 1000)

    return ResponsePaginator(
        client,
        url=f'/album-comment-mobile/web/album/comment/list/query/{timestamp}',
        params={'albumId': album_id, 'order': order, 'pageSize': page_size},
        item_class=AlbumComment,
        data_path='data.comments.list',
        page_key='pageId'
    )



from ximalaya.core.client import XimalayaClient, ResponsePaginator
from ximalaya.core.models import FollowingInfo, UserBasicInfo, UserDetailedInfo, SubscriptionInfo


def api_user_following(client: XimalayaClient, *, uid: int, page_size: int = 50) -> ResponsePaginator[FollowingInfo]:
    client.host = 'www.ximalaya.com'

    return ResponsePaginator(
        client,
        url=f'/revision/user/following',
        params={'uid': uid, 'pageSize': page_size},
        item_class=FollowingInfo,
        data_path='data.followingsPageInfo'
    )


def api_user_basic(client: XimalayaClient, *, uid: int) -> UserBasicInfo:
    client.host = 'www.ximalaya.com'

    return UserBasicInfo(**client.get(f'/revision/user/basic?uid={uid}')['data'])


def api_user(client: XimalayaClient, *, uid: int) -> UserDetailedInfo:
    client.host = 'www.ximalaya.com'

    return UserDetailedInfo(**client.get(f'/revision/user?uid={uid}')['data'])


def api_user_sub(client: XimalayaClient, *, uid: int, page_size: int = 10) -> ResponsePaginator[SubscriptionInfo]:
    client.host = 'www.ximalaya.com'

    return ResponsePaginator(
        client,
        url='/revision/user/sub',
        params={'uid': uid, 'pageSize': page_size},
        item_class=SubscriptionInfo,
        data_path='data.albumsInfo'
    )



import logging
from typing import Generic, Iterator, Type, TypeVar

import requests
from jsonpath_ng import parse


class XimalayaClient:
    host: str

    def __init__(self, host: str = 'www.ximalaya.com', *, headers: dict = None):
        self.host = host

        self.headers = {} if headers is None else headers

        if 'User-Agent' not in self.headers:
            self.headers['User-Agent'] = ''

    def get(self, path: str, *, params: dict = None) -> dict:
        url = f'https://{self.host}/{path.lstrip("/")}'

        if params is None:
            params = {}

        logging.debug('[XimalayaClient][GET]: %s', url)

        r = requests.get(url=url, headers=self.headers, params=params)

        if r.status_code != 200:
            raise requests.HTTPError(r.content.decode('utf-8'))

        return r.json()


T = TypeVar('T')


class ResponsePaginator(Generic[T], Iterator):
    client: XimalayaClient
    url: str
    item_class: Type[T]
    params: dict
    page_key: str
    page_num: int

    def __init__(self,
                 client: XimalayaClient,
                 *,
                 url: str,
                 params: dict = None,
                 data_path: str,
                 item_class: Type[T],
                 page_key: str = 'page',
                 page_num: int = 1):
        self.client = client
        self.url = url
        self.params = {} if params is None else params
        self.item_class = item_class
        self.page_key = page_key
        self.page_num = page_num
        self.jsonpath_expression = parse(f'$.{data_path}')

    def __next__(self) -> list[T]:
        json_data = self.client.get(self.url, params={self.page_key: self.page_num, **self.params})

        match = self.jsonpath_expression.find(json_data)

        if match is None or len(match[0].value) == 0:
            raise StopIteration()

        self.page_num += 1

        return [self.item_class(**item) for item in match[0].value]



from ximalaya.core.client import XimalayaClient, ResponsePaginator
from ximalaya.core.models import AlbumInfo


def api_category_v2_albums(client: XimalayaClient, *, category_id: int, page_size: int = 50, sort_by: int = 1) -> ResponsePaginator[AlbumInfo]:
    client.host = 'www.ximalaya.com'

    return ResponsePaginator(
        client,
        url='/revision/category/v2/albums',
        params={'pageSize': page_size, 'sort': sort_by, 'categoryId': category_id},
        item_class=AlbumInfo,
        page_key='pageNum',
        data_path='data.albums'
    )



from __future__ import annotations

from pydantic import BaseModel
from typing import Optional


class AlbumComment(BaseModel):
    albumId: int
    albumPaid: bool
    albumUid: int
    anchorLiked: bool
    auditStatus: int
    commentId: int
    content: str
    createdAt: int
    isHighQuality: bool
    likeStatus: Optional[int] = 0
    liked: Optional[bool] = False
    likes: int
    nickname: str
    playProgress: int
    recStatus: Optional[int] = 0
    region: str
    replyCount: int
    smallHeader: str
    uid: int
    updatedAt: int
    userTags: list[UserTag]
    vLogoType: int
    vipIconUrl: Optional[str] = None
    vipJumpUrl: Optional[str] = None


class AlbumInfo(BaseModel):
    albumId: int
    albumPlayCount: int
    albumTrackCount: int
    albumCoverPath: str
    albumTitle: str
    albumUserNickName: str
    anchorId: int
    anchorGrade: int
    mvpGrade: int
    isDeleted: bool
    isPaid: bool
    isFinished: int
    anchorUrl: str
    albumUrl: str
    intro: str
    vipType: int
    logoType: int
    subscriptInfo: SubscriptInfo
    albumSubscript: int


class AnchorInfo(BaseModel):
    anchorUrl: str
    anchorNickName: str
    anchorUid: int
    anchorCoverPath: str
    logoType: int


class FollowingInfo(BaseModel):
    uid: int
    coverPath: str
    anchorNickName: str
    background: str
    description: Optional[str] = None
    url: str
    grade: int
    mvpGrade: int
    gradeType: int
    trackCount: int
    albumCount: int
    followerCount: int
    followingCount: int
    isFollow: bool
    beFollow: bool
    isBlack: bool
    logoType: int


class FollowingPageInfo(BaseModel):
    totalCount: int
    followInfoList: list[FollowingInfo]


class LiveInfo(BaseModel):
    id: int = -1
    roomId: Optional[int] = None
    coverPath: Optional[str] = None
    liveUrl: Optional[str] = None
    status: Optional[int] = None


class PubInfo(BaseModel):
    id: int
    title: str
    subTitle: str
    coverPath: str
    isFinished: bool
    isPaid: bool
    anchorUrl: str
    anchorNickname: Optional[str] = None
    anchorUid: int
    playCount: int
    trackCount: int
    albumUrl: str
    description: str
    vipType: int
    albumSubscript: int


class PubPageInfo(BaseModel):
    totalCount: int
    pubInfoList: list[PubInfo]


class QualificationGuideInfo(BaseModel):
    guideMsg: str
    guideLinkUrl: str


class SubscriptInfo(BaseModel):
    albumSubscriptValue: int
    url: str


class SubscriptionInfo(BaseModel):
    id: int
    title: str
    subTitle: str
    description: str
    coverPath: str
    isFinished: bool
    isPaid: bool
    anchor: AnchorInfo
    playCount: int
    trackCount: int
    albumUrl: str
    albumStatus: int
    lastUptrackAt: int
    lastUptrackAtStr: str
    serialState: int
    isTop: bool
    categoryCode: str
    categoryTitle: str
    lastUptrackUrl: str
    lastUptrackTitle: str
    vipType: int
    albumSubscript: int
    albumScore: str


class SubscriptionPageInfo(BaseModel):
    privateSub: bool
    totalCount: int
    subscribeInfoList: list[SubscriptionInfo]


class TrackInfo(BaseModel):
    trackId: int
    title: str
    trackUrl: str
    coverPath: str
    createTimeAsString: str
    albumId: int
    albumTitle: str
    albumUrl: str
    anchorUid: int
    anchorUrl: str
    nickname: str
    durationAsString: str
    playCount: int
    showLikeBtn: bool
    isLike: bool
    isPaid: bool
    isRelay: bool
    showDownloadBtn: bool
    showCommentBtn: bool
    showForwardBtn: bool
    isVideo: bool
    videoCover: str
    breakSecond: int
    length: int
    isAlbumShow: bool


class TrackPageInfo(BaseModel):
    totalCount: int
    trackInfoList: list[TrackInfo]


class UserBasicInfo(BaseModel):
    uid: int
    nickName: str
    cover: str
    background: str
    isVip: bool
    constellationType: int
    personalSignature: Optional[str] = None
    personalDescription: Optional[str] = None
    fansCount: int
    gender: int
    birthMonth: int
    birthDay: int
    province: Optional[str] = None
    city: Optional[str] = None
    anchorGrade: int
    mvpGrade: int
    anchorGradeType: int
    isMusician: bool
    anchorUrl: str
    relation: UserRelation | None
    liveInfo: LiveInfo
    logoType: int
    followingCount: int
    tracksCount: int
    albumsCount: int
    albumCountReal: int
    userCompany: str
    qualificationGuideInfos: list[QualificationGuideInfo]


class UserDetailedInfo(BaseModel):
    uid: int
    pubPageInfo: PubPageInfo
    trackPageInfo: TrackPageInfo
    subscriptionPageInfo: SubscriptionPageInfo
    followingPageInfo: FollowingPageInfo


class UserRelation(BaseModel):
    isFollow: bool
    beFollow: bool
    isBlack: bool


class UserTag(BaseModel):
    businessName: str
    businessType: int
    extra: dict
    height: int
    icon: str
    jumpUrl: str
    scale: int
    width: int
